/***************************************************************************
                           filesidestyle.cpp
                           -------------------
    copyright            : (C) 2004 by Jos van den Oever
    email                : jos@vandenoever.info
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "filesidestyle.h"
#include "testdialog.h"
#include <QDir>
#include <QPicture>
#include <QPictureIO>
#include <Q3Picture> // needed for SVG support
#include <QPixmap>
#include <QImage>
#include <QPainter>
#include <QApplication>
#include <QImageReader>
#include <iostream>
using std::cerr;
using std::endl;

/* TODO
   - add check for identical files
   - add check for bitmap symmetry
*/


FileSideStyle::FileSideStyle() {
	readDirs(QDir::homePath()+"/.cubetest/styles");
	readDirs(qApp->applicationDirPath()+"/../share/cubetest/styles");
	QStringList sl = styledirs.keys();
}
FileSideStyle::~FileSideStyle() {
	freeimages();
}
void
FileSideStyle::freeimages() {
	std::vector<const QPicture *>::const_iterator i;
	for (i=c1.begin(); i != c1.end(); ++i) {
		delete *i;
	}
	c1.clear();
	for (i=c2.begin(); i != c2.end(); ++i) {
		delete *i;
	}
	c2.clear();
	for (i=c4.begin(); i != c4.end(); ++i) {
		delete *i;
	}
	c4.clear();
}
void
FileSideStyle::readDirs(const QString &dir) {
	QDir d(dir);
	if (!d.exists()) return;
	foreach(QFileInfo fi, d.entryInfoList(QDir::Dirs)) {
		QString newdir = fi.absoluteFilePath();
		QString newname = fi.fileName();
		if (!styledirs.contains(newname) && checkDir(newdir)) {
			styledirs[newname] = newdir;
		}
        }
}
bool
FileSideStyle::checkDir(const QDir &d) const {
	// a valid dir has at least one C4, two of either C1 or C2
	bool hasC4 = false;
	short other = 0;
	Q3Picture svgloader;
	foreach(QFileInfo fi, d.entryInfoList(QDir::Files)) {
		QString path = fi.absoluteFilePath();
		// check if the file is a raster image
		QImageReader img(path);
		bool isimage = img.canRead();
		if (!isimage) {
			if (fi.fileName().right(4).toLower() == ".svg") {
				// check if the file is a vector image and can
				// be read. Qt4 does not yet support SVG so we
				// use Q3Picture
				isimage = svgloader.load(path, "svg");
			} else {
				foreach (QByteArray ba,
						QPictureIO::inputFormats()) {
					QPictureIO pio(path, ba);
					isimage = pio.read();
					if (isimage) {
						break;
					}
				}
			}
		}
		if (isimage) {
			QString begin = fi.fileName().left(2).toLower();
			if (begin == "c4") {
				hasC4 = true;
			} else if ( (begin == "c1" || begin == "c2")) {
				++other;
			}
			if (hasC4 && other > 1) {
				break;
			}
		} else {
		}
        }
	return hasC4 && other > 1;
}
void
FileSideStyle::loadStyle(const QString &s) {
	if (styledirs.contains(s)) {
		freeimages();
		loadDir(styledirs[s]);
	}
}
void
FileSideStyle::loadDir(const QDir &d) {
	c1.clear();
	c2.clear();
	c4.clear();
	foreach(QFileInfo fi, d.entryInfoList(QDir::Files)) {
		loadFile(fi.absoluteFilePath());
	}
}
Symmetry
getSym(const QImage &img) {
	QMatrix wm180, wm90;
	wm180.rotate(180);
	wm90.rotate(90);
	if (img != img.transformed(wm180)) {
		return C1;
	} else if (img != img.transformed(wm90)) {
		return C2;
	} else {
		return C4;
	}
}
QImage
makeImage(const QPicture p, int res) {
	QRect r = p.boundingRect();
	r.setWidth(r.width());
	r.setHeight(r.height());
	QPixmap pm(res, res);
	pm.fill(Qt::white);
	QPainter painter;
	painter.begin(&pm);
	painter.setWindow(r);
	painter.scale(res/r.width(), res/r.height());
	painter.translate(r.width()/2, r.height()/2);
	painter.drawPicture(r.x(),r.y(), p);
	painter.end();
	return pm.toImage();
}
void
FileSideStyle::loadFile(const QString &f) {
	QPicture p;
	QPainter painter;
	QPicture *pict;
	QImageReader img(f);
	bool pix = img.canRead();
	if (pix) {
		QImage img(f);
		painter.begin(&p);
		painter.fillRect(0,0,img.width(),img.height(),Qt::white);
		painter.drawImage(0,0,img);
		painter.end();
		pict = new QPicture(p);
	} else {
		if (f.right(4).toLower() == ".svg") {
			Q3Picture p3;
			if (!p3.load(f, "svg")) {
				return;
			}
			p = p3;
		} else {
			QPictureIO pio;
			pio.setFileName(f);
			if (!pio.read()) {
				return;
			}
			p = pio.picture();
		}
		QRect r = p.boundingRect();
		pict = new QPicture();
		painter.begin(pict);
		painter.fillRect(r, Qt::white);
		painter.drawPicture(0,0,p);
		painter.end();
	}
	if (f.contains("C1_", Qt::CaseInsensitive)) {
		c1.push_back(pict);
	} else if (f.contains("C2_", Qt::CaseInsensitive)) {
		c2.push_back(pict);
	} else {
		if (c4.size() > 0 && f.contains("c4_blank",
				Qt::CaseInsensitive)) {
			c4.push_back(c4[0]);
			c4[0] = pict;
		} else {
			c4.push_back(pict);
		}
	}
}
QStringList
FileSideStyle::getStyles() {
	return styledirs.keys();
}
const QPicture *
FileSideStyle::getC4Picture(ushort i) const {
	if (i < c4.size()) return c4[i];
	cerr << "index " << i << " is out of range in getC4Picture" << endl;
	throw 0;
}
const QPicture *
FileSideStyle::getC2Picture(ushort i) const {
	if (i < c2.size()) return c2[i];
	cerr << "index " << i << " is out of range in getC2Picture" << endl;
	throw 0;
}
const QPicture *
FileSideStyle::getC1Picture(ushort i) const {
	if (i < c1.size()) return c1[i];
	cerr << "index " << i << " is out of range in getC1Picture" << endl;
	throw 0;
}
short
FileSideStyle::amount(Symmetry s) const {
	if (s == C1) return c1.size();
	if (s == C2) return c2.size();
	if (s == C4) return c4.size();
	return 0;
}
