/***************************************************************************
                           object.cpp
                           -------------------
    copyright            : (C) 2004 by Jos van den Oever
    email                : jos@vandenoever.info
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "object.h"
#include <QPicture>
#include <QPainter>
#include <QTimer>
//Added by qt3to4:
#include <QPixmap>
#include <QMouseEvent>
#include <QResizeEvent>
#include <QPaintEvent>
#include <cmath>
#include <iostream>

Object::Object(QWidget *parent)
        : QWidget(parent) {

	scaling = 1;
	m.scale(scaling);
	setMouseTracking(true);
	firstMouseEvent = true;
	changed = true;
	maxdist=0;
	elements=0;

	currentCursor = 0;
	setCursor(Qt::SizeHorCursor);
	rotangle = 0;

	lastUpdate.start();
}
void
Object::addSide(Side s) {
	double newdist;
	newdist = s.getMaxDistance();
	if (newdist > maxdist) {
		maxdist = newdist;
		m.scale(1/scaling);
		scaling = 2*maxdist;
		m.scale(scaling);
	}
	s.number = elements++;
	sides.push_back(s);
}
const Side &
Object::getSide(short i) const {
	return sides[i];
}
Side &
Object::getSide(short i) {
	return sides[i];
}
bool
Object::hasPicture(const QPicture *p1, short i) const {
	QList<Side>::const_iterator it = sides.begin();
	while (it != sides.end()) {
		const QPicture *p2 = (*it).picture();
		if ((*it).number != i && p1->size() == p2->size()
			&& !strncmp(p1->data(), p2->data(), p1->size())) {
			return true;
		}
		it++;
	}
	return false;
}
void
Object::rotateX(double deg) {
	m.rotX(deg);
	changed = true;
	emit rotatedX(deg);
	emit rotated();
	callUpdate();
}
void
Object::rotateY(double deg) {
	m.rotY(deg);
	changed = true;
	emit rotatedY(deg);
	emit rotated();
	callUpdate();
}
void
Object::rotateZ(double deg) {
	m.rotZ(deg);
	changed = true;
	emit rotated();
	callUpdate();
}
void
Object::setOrient(double t, double p, double r) {
	m.setOrient(t, p, r);
	changed = true;
	callUpdate();
}
void
Object::reset() {
	m.unit();
	m.savePosition();
	m.scale(scaling);
	changed = true;
	callUpdate();
}
/**
 * Throttle the amount of updates that are possible per second.
 **/
void
Object::callUpdate() {
	if (lastUpdate.elapsed() > 50) {
		update();
	} else if (!updateTimer.isActive()) {
		updateTimer.singleShot(50, this, SLOT(update()));
	}
	lastUpdate.restart();
}
void
Object::paintEvent(QPaintEvent *) {
	updateMatrices();
	QPainter painter(this);
//	painter.setRenderHint(QPainter::Antialiasing, true);
	QPixmap background = palette().background().texture();
	if (!background.isNull()) {
		QSize s = background.size();
		QPoint p(0,0);
		p = mapTo(topLevelWidget(), p);
		p.rx() %= s.width();
		p.ry() %= s.height();
		painter.drawTiledPixmap(rect(), background, p);
	} else {
		painter.fillRect(rect(), palette().background().color());
	}
	painter.translate(width()/2,height()/2);
	painter.setPen(Qt::black);
	QList<Side>::const_iterator it;
	it = sides.begin();
	while (it != sides.end()) {
		(*it).paintSide(&painter);
		it++;
	}
}
void
Object::mouseMoveEvent ( QMouseEvent * e ) {
	int x = e->x();
	int y = e->y();
	if (!firstMouseEvent && e->buttons() == Qt::LeftButton) {
		switch (rotangle) {
		case 0:
			m.rotY(x-lastX);
			break;
		case 1:
			m.rotX(y-lastY);
			break;
		case 2:
			m.rotZ(x-lastX);
			updateCursor(x, y, true);
			break;
		case 3:
			m.rotZ(lastY-y);
			updateCursor(x, y, true);
			break;
		case 4:
			m.rotZ(lastX-x);
			updateCursor(x, y, true);
			break;
		case 5:
			m.rotZ(y-lastY);
			updateCursor(x, y, true);
			break;
		}
		emit rotated();
		changed = true;
		callUpdate();
		movedSinceMousePress = true;
	} else {
		updateCursor(x,y);
	}
	lastX = x;
	lastY = y;
	firstMouseEvent = false;
}
void
Object::mousePressEvent(QMouseEvent *e) {
	if (e->buttons() == Qt::LeftButton) {
		movedSinceMousePress = false;
	}
}
void
Object::mouseReleaseEvent(QMouseEvent *e) {
	if (e->buttons() == Qt::LeftButton) {
		int x = e->x();
		int y = e->y();
		QPoint p;
		if (!movedSinceMousePress) {
			p.setX(x-width()/2);
			p.setY(y-height()/2);
			int i = sides.size()-1;
			do {
				if (sides[i].processClick(p)) {
					break;
				}
			} while (i--);
			changed = true;
			callUpdate();
		}
		updateCursor(x,y);
	}
}
/* The maximal size of the the cube is 2*sqrt(3). So, the matrix should be scaled min(width, height)/2/sqrt(3). */
void
Object::resizeEvent( QResizeEvent * ) {
	double dim = (width()<height())?width():height();
	m.scale(1/scaling);
	scaling = dim/2.0/maxdist;
	m.scale(scaling);
	changed = true;
}
void
Object::updateMatrices() {
	QList<Side>::iterator it;
	it = sides.begin();
	while (it != sides.end()) {
		(*it).updateMatrix(m);
		it++;
	}
/*
	class Compare_sides {
	public:
		int operator()(pair<Side, short int> s1, pair<Side, short int> s2) {
			return s1.first < s2.first;}
	};
	sides.sort(Compare_sides());
*/
	qSort(sides);
}
void
Object::animateToSavedOrientation() {
	animate = true;
        QTimer::singleShot(40, this, SLOT(animationStep()));
}
void
Object::animationStep() {
	if (animate && m.stepBack()) {
		changed = true;
		callUpdate();
		QTimer::singleShot(40, this, SLOT(animationStep()));
	} else {
		emit animationFinished();
	}
}
void
Object::stopAnimation() {
	animate = false;
}
void
Object::saveOrientation() {
	m.savePosition();
}
void
Object::setPicture(short i, const QPicture *p) {
	getSide(i).setPicture(p);
}
/* The cursor changes depending on the position above the object.
   Also, the rotation depends on the starting position above the object.

	\2222222/
	3\22122/5
	33\111/55
	330\1/055
	3000X0005
	330/1\055
	33/111\55
	3/44144\5
	/4444444\

   The value of 0 means rotation around the y axis, 1 means rotation around the
   x axis and the other value mean rotation around the z axis. */
void
Object::updateCursor(int x, int y, bool onlyZ) {
	float fx, fy;
	float ax, ay;
	fx = float(x)/width();
	fy = float(y)/height();
	ax = fabs(fx-0.5);
	ay = fabs(fy-0.5);
	if (ax > ay) {
		if (fx < 0.5) {
			rotangle = 1;
		} else {
			rotangle = 3;
		}
	} else {
		if (fy < 0.5) {
			rotangle = 0;
		} else {
			rotangle = 2;
		}
	}
	if (!onlyZ && ax*ax+ay*ay < 0.1) {
		if (rotangle == 1 || rotangle == 3) {
			rotangle = 0;
		} else {
			rotangle = 1;
		}
	} else {
		rotangle += 2;
	}

	int newcursor = rotangle%2;
	if (currentCursor != newcursor) {
		if (newcursor == 0) {
			setCursor(Qt::SizeHorCursor);
		} else {
			setCursor(Qt::SizeVerCursor);
		}
		currentCursor = newcursor;
	}
}
